include "Vram.inc"
include "VVram.inc"

ext Chars_, Ascii_
ext VVramFront_, RowFlags_

VramTop equ Vram+VramRowSize

ReadPlain equ 0f1h
WritePlain equ 0f2h

dseg
Backup:
    defs VVramWidth*VVramHeight


; void InitColors();
cseg
InitColors_: public InitColors_
    push af
        ld a,44h
        out (0f5h),a
        out (0f8h),a
        ld a,11h
        out (0f6h),a
        out (0f9h),a
        ld a,22h
        out (0f7h),a
        out (0fah),a
    pop af
ret


cseg
ClearRowFlags:
    xor a
    ld hl,RowFlags_
    ld b,4
    do
        ld (hl),a
        inc hl
    dwnz
ret


; void ClearScreen();
cseg
ClearScreen_: public ClearScreen_
    push af | push hl | push de | push bc
        ld a,07h
        out (WritePlain),a

        ld hl,Vram
        ld de,Vram+1
        ld bc,VramRowSize*VramHeight-1
        ld (hl),0
        ldir

        ld hl,Backup
        ld de,Backup+1
        ld bc,VVramWidth*VVramHeight-1
        ld (hl),0
        ldir

        call ClearRowFlags
    pop bc | pop de | pop hl | pop af
ret


PatternAddress:
    push de
        ld l,a
        ld h,0
        ld e,l
        ld d,h
        add hl,hl ;*2
        add hl,de ;*3
        add hl,hl ;*6
        add hl,hl ;*12
        add hl,hl ;*24
        ld de,Chars_
        add hl,de
    pop de
ret


; void VPut(word vram, byte c);
VPut_: public VPut_
    push af | push hl | push de | push bc
        ld a,e
        ex de,hl
            call PatternAddress
        ex de,hl
        ld bc,0301h
        do
            ld a,c | out (WritePlain),a
            push bc
                ld b,CharHeight
                do
                    ld a,(de) | inc de
                    ld (hl),a
                    push bc
                        ld bc,VramWidth
                        add hl,bc
                    pop bc
                dwnz
                ld bc, -VramWidth*CharHeight
                add hl,bc
            pop bc
            sla c
        dwnz
    pop bc | pop de | pop hl | pop af
ret


dseg
PlainBit:
    defb 0
PlainIndex:
    defb 0

cseg
PutShift:
    ex de,hl
        call PatternAddress
    ex de,hl
    ld a,(PlainBit)
    ld c,a                       
    ld b,0
    do
        ld a,c
        out (WritePlain),a
        ld a,(PlainIndex)
        add a,b
        out (ReadPlain),a
        push bc
            ld b,CharHeight
            do
                push bc
                    ld a,(de) | inc de
                    push de
                        ld e,a | ld d,0
                        ld a,i
                        ld bc,0ffc0h
                        or a
                        do
                        while nz
                            sla e | rl d
                            sla c | rl b
                            set 0,c
                            dec a
                        wend
                        ld a,(hl)
                        and c
                        or e
                        ld (hl),a
                        inc hl
                        ld a,(hl)
                        and b
                        or d
                        ld (hl),a

                        push bc
                            ld bc,VramWidth-1
                            add hl,bc
                        pop bc
                    pop de
                pop bc
            dwnz
            ld bc, -VramWidth*CharHeight
            add hl,bc
        pop bc
        sla c
        inc b
        ld a,b
        cp 3
    while nz | wend
ret


cseg
VVramToVramRow:
    push bc
        ld bc,VVramWidth*100h
        do 
            push bc 
                ld a,(de)
                inc de
                cp (hl)
                if nz
                    ld (hl),a
                    ex af,af'
                        ld a,c
                        ld i,a
                    ex af,af'
                    exx
                        call PutShift
                    exx
                endif
                inc hl
            pop bc
            ld a,c
            add a,6
            cp 8
            if nc
                exx
                    inc hl
                exx
                sub 8
            endif
            ld c,a
        dwnz
        exx
            push bc
                ld bc,VramRowSize-VramWidth
                add hl,bc
            pop bc
        exx
    pop bc
ret


; void VVramToVramAll();
VVramToVramAll_: public VVramToVramAll_
    push af | push hl | push de | push bc
        ld de,VVramFront_+VVramWidth
        ld hl,Backup
        exx
            ld hl,VramTop
        exx
        ld a,1 | ld (PlainBit),a
        ld (PlainIndex),a
        
        ld c, VVramHeight-2
        do
            call VVramToVramRow
            dec c
        while nz | wend
        call ClearRowFlags
    pop bc | pop de | pop hl | pop af
ret


; void VVramToVramChanged();
cseg
VVramToVramChanged_: public VVramToVramChanged_
    push af | push hl | push de | push bc
        ld de,VVramFront_+VVramWidth
        ld hl,Backup
        exx
            ld hl,VramTop
            ld de,RowFlags_
            ld c,2
            ld a,(de) | inc de
            ld b,a
        exx
        ld c, VVramHeight-2
        do
            exx
                ld a,b
                and c
            exx
            if nz
                exx
                    push de | push bc
                exx
                call VVramToVramRow            
                exx
                    pop bc | pop de
                exx
            else
                push bc
                    ld bc,VVramWidth
                    add hl,bc
                    ex de,hl
                        add hl,bc
                    ex de,hl
                pop bc
                exx
                    push bc
                        ld bc,VramRowSize
                        add hl,bc
                    pop bc
                exx
            endif
            exx
                rlc c
                if c
                    ld a,(de) | inc de
                    ld b,a
                endif
            exx
            dec c
        while nz | wend
        call ClearRowFlags
    pop bc | pop de | pop hl | pop af
ret


; void VPutShift(word vram, byte xMod, byte c);
cseg
VPutShift_: public VPutShift_
    push af | push hl | push de | push bc
        ld a,1 | ld (PlainBit),a
        ld a,1 | ld (PlainIndex),a
        ld a,e
        ld i,a
        ld a,c
        call PutShift
    pop bc | pop de | pop hl | pop af
ret


dseg
CurrentVram:
    defw 0
CharShift:
    defb 0
; TextPlain_: public TextPlain_
;     defb 0

; void Locate(word vram, bool shift);
cseg
Locate_: public Locate_
    push af
        ld (CurrentVram),hl
        ld a,e
        ld (CharShift),a
    pop af
ret


; void PrintC(byte c);
cseg
PrintC_: public PrintC_
    push af | push hl | push de | push bc
        sub ' '
        ld l,a
        ld h,0
        add hl,hl ;*2
        add hl,hl ;*4
        ld de,Ascii_
        add hl,de
        ex de,hl

        ld a,07h | out (WritePlain),a
        ld a,1 | out (ReadPlain),a
        ; ld h,a
        ; ld l,1
        ; ld c,1
        ; do
        ;     ld a,l
        ;     and h
        ;     if nz
        ;         ld a,c
        ;         out (ReadPlain),a
        ;         jr PrintC_break
        ;     endif
        ;     sla l
        ;     inc c
        ;     ld a,c
        ;     cp 6
        ; while nz | wend
        ; PrintC_break:

        ld hl,(CurrentVram)
        ld a,(CharShift)
        or a
        if z
            ld b,4
            do
                push bc
                    ld a,(de) | inc de
                    ld c,a
                    push de
                        ld de,VramWidth

                        and 0fh
                        ld b,a
                        ld a,(hl)
                        and 0f0h
                        or b
                        ld (hl),a
                        add hl,de

                        ld a,c
                        rrca | rrca | rrca | rrca
                        and 0fh
                        ld b,a
                        ld a,(hl)
                        and 0f0h
                        or b
                        ld (hl),a
                        add hl,de
                    pop de
                pop bc
            dwnz
        else
            ld b,4
            do
                push bc
                    ld a,(de) | inc de
                    ld c,a
                    push de
                        ld de,VramWidth

                        rlca | rlca | rlca | rlca
                        and 0f0h
                        ld b,a
                        ld a,(hl)
                        and 0fh
                        or b
                        ld (hl),a
                        add hl,de

                        ld a,c
                        and 0f0h
                        ld b,a
                        ld a,(hl)
                        and 0fh
                        or b
                        ld (hl),a
                        add hl,de
                    pop de
                pop bc
            dwnz
        endif
        ld a,(CharShift)
        xor 1
        ld (CharShift),a
        if z
            ld hl,(CurrentVram)
            inc hl
            ld (CurrentVram),hl
        endif
    pop bc | pop de | pop hl | pop af
ret
